//
//  BlockMatrixWrapper.h
//  SparseBlockMatrix
//
//  Created by Daniel Vollmer on 18/03/2013.
//  Copyright (c) 2013 DLR. All rights reserved.
//

#ifndef SparseBlockMatrix_BlockMatrixWrapper_h
#define SparseBlockMatrix_BlockMatrixWrapper_h

#include "Eigen/Dense"
#include "Eigen/Sparse"

#define BMW_DEBUG

template<typename _Matrix>
class BlockMatrixWrapper : public _Matrix
{
public:
	enum
	{
		BlockRowsAtCompileTime = _Matrix::RowsAtCompileTime,
		BlockColsAtCompileTime = _Matrix::ColsAtCompileTime
	};
	
	BlockMatrixWrapper(void):_Matrix() {}
	
	// comment to see who tries to create instances via this constructor
	BlockMatrixWrapper(typename _Matrix::Scalar V)
	: _Matrix(_Matrix::Zero())
	{
#ifdef BMW_DEBUG
		std::cout << "BMW(" << V << ") "; // shows when we get constructed with BlockMatrixWrapper(0) for example
#endif
		_Matrix::diagonal().setConstant(V);
	}
	
	// This constructor allows you to construct BlockMatrix from Eigen expressions
	template<typename OtherDerived>
	BlockMatrixWrapper(const Eigen::MatrixBase<OtherDerived>& other)
	: _Matrix(other)
	{
	}
	
	// This method allows you to assign Eigen expressions to BlockMatrix
	template<typename OtherDerived>
	BlockMatrixWrapper& operator=(const Eigen::MatrixBase <OtherDerived>& other)
	{
		_Matrix::operator=(other);
		return *this;
	}
};

// needed for InnerIterator :(
template<typename _Matrix>
inline typename Eigen::NumTraits<typename _Matrix::Scalar>::Real abs(const BlockMatrixWrapper<_Matrix>& x)
{
#ifdef BMW_DEBUG
	std::cout << "BMW::abs( " << x << ") ";
#endif
	return x.norm();
}

namespace Eigen
{
	template<typename _Matrix> struct NumTraits<BlockMatrixWrapper<_Matrix> >
	: NumTraits<typename _Matrix::Scalar>
	{
		typedef typename NumTraits<typename _Matrix::Scalar>::Real Real;
		typedef _Matrix NonInteger;
		typedef _Matrix Nested;
		enum
		{
			RequireInitialization = 1,
			ReadCost = _Matrix::RowsAtCompileTime * _Matrix::ColsAtCompileTime * NumTraits<typename _Matrix::Scalar>::ReadCost,
			AddCost = _Matrix::RowsAtCompileTime * _Matrix::ColsAtCompileTime * NumTraits<typename _Matrix::Scalar>::AddCost,
			MulCost = _Matrix::RowsAtCompileTime * _Matrix::ColsAtCompileTime * _Matrix::ColsAtCompileTime * NumTraits<typename _Matrix::Scalar>::MulCost
			+ _Matrix::RowsAtCompileTime * _Matrix::ColsAtCompileTime * (_Matrix::ColsAtCompileTime-1) * NumTraits<typename _Matrix::Scalar>::AddCost
		};
	};
	
	namespace internal
	{
		template<typename BMA, typename BMB>
		struct scalar_product_traits<BlockMatrixWrapper<BMA>, BlockMatrixWrapper<BMB> >
		{
			typedef BlockMatrixWrapper<Eigen::Matrix<typename scalar_product_traits<typename BMA::Scalar, typename BMB::Scalar>::ReturnType, BMA::RowsAtCompileTime, BMB::ColsAtCompileTime> > ReturnType;
		};
	};
}

#endif
