//
//  main.cpp
//  SparseMatrixTest
//
//  Created by Daniel Vollmer on 15/03/2013.
//  Copyright (c) 2013 DLR. All rights reserved.
//

#include <iostream>

#include "BlockMatrixWrapper.h"
//#include "Eigen/SparseLU"
//#include "Eigen/IterativeLinearSolvers"

typedef Eigen::MatrixXd DenseMatrix;
typedef BlockMatrixWrapper<Eigen::Matrix<double, 5, 5> > BlockMatrixScalar;
typedef BlockMatrixWrapper<Eigen::Matrix<double, 5, 1> > BlockVectorScalar;

typedef Eigen::SparseMatrix<BlockMatrixScalar> SparseBlockMatrix;
typedef Eigen::Matrix<BlockVectorScalar, Eigen::Dynamic, 1> BlockVector;


DenseMatrix ExpandSparseBlockMatrix(const SparseBlockMatrix &s)
{
	DenseMatrix result(s.rows() * SparseBlockMatrix::Scalar::BlockRowsAtCompileTime, s.cols() * SparseBlockMatrix::Scalar::BlockColsAtCompileTime);
	
	result.setZero();
	for (SparseBlockMatrix::Index k = 0; k < s.outerSize(); k++)
	{
		for (SparseBlockMatrix::InnerIterator it(s, k); it; ++it)
		{
			result.block<SparseBlockMatrix::Scalar::BlockRowsAtCompileTime, SparseBlockMatrix::Scalar::BlockColsAtCompileTime>(it.row() * SparseBlockMatrix::Scalar::BlockRowsAtCompileTime, it.col() * SparseBlockMatrix::Scalar::BlockColsAtCompileTime) = it.value();
		}
	}
	return result;
}

int main(int argc, const char * argv[])
{
	const int size = 3;
	SparseBlockMatrix sm(size, size);
	BlockMatrixScalar block(BlockMatrixScalar::Zero());
	BlockVector rhs(size), soln(size);
	
	block.diagonal().setOnes();
	for (int i = 0; i < size; i++)
	{
		sm.insert(i, i) = block;
		if (i > 0)
		{
			sm.insert(i, i - 1) = block;
		}
		if (i < size - 1)
		{
			sm.insert(i, i + 1) = block;
		}
		rhs[i].setOnes();
	}
//	sm.makeCompressed();
	DenseMatrix dense = ExpandSparseBlockMatrix(sm);
	
	sm = sm * sm; // SparseMatrix doesn't seem to have operator*=
	dense *= dense;
	std::cout << "ErrorNorm: " << (dense - ExpandSparseBlockMatrix(sm)).norm() << std::endl << dense << std::endl;
	
//	Eigen::ConjugateGradient<SparseBlockMatrix> solver;
//	solver.compute(sm);
//	soln = solver.solve(rhs);
	
	const BlockMatrixScalar &a = sm.sum();
	std::cout << "nonZeros: " << sm.nonZeros() << std::endl << a << std::endl;
	
	soln = sm * rhs;
	std::cout << "prod1: " << soln << std::endl;
    return 0;
}

